<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Util\Procedure\FctSincronizarComposicao;
use Teknisa\SM\Factory\Sm00006000ComposicaoServicosFactory as ComposicaoFactory;

class Sm00006000ComposicaoServicosService {
    
    protected $connection;
    protected $composicaoFactory;
    
    public function __construct(EntityManager $entityManager, ComposicaoFactory $composicaoFactory) {
        $this->connection        = $entityManager->getConnection();
        $this->composicaoFactory = $composicaoFactory;
    }
    
    /**
     * Executa a procedure FCT_SINCRONIZAR_COMPOSICAO.
     * 
     * @param integer $nrcompservico
     * @param integer $nrorg
     */
    public function executarProcedureFctSincronizarComposicao($nrcompservico, $nrorg) {
        $procedure = new FctSincronizarComposicao($this->connection);
        $procedure->executar($nrcompservico, $nrorg);
    }
    
    /**
     * Exclui a composição e seus filhos.
     * 
     * @param integer $nritcompservico
     * @param integer $nrorg
     */
    public function excluirComposicaoEmCascata($nrcompservico, $nrorg) {
        $params = array(
            "NRCOMPSERVICO" => $nrcompservico,
            "NRORG"         => $nrorg
        );
        
        $this->composicaoFactory->excluirComposicaoEmCascata($params);
    }
    /**
     * Replica o IDATIVO para todos os filhos da composição de serviço  alterada
    **/
    public function replicarAlteracaoIdAtivo($idAtivo, $nrcompservico, $operador, $nrorg) {
        $params = array(
            "IDATIVO"       => $idAtivo,
            "NRCOMPSERVICO" => $nrcompservico,
            "CDOPERADOR"    => $operador,
            "NRORG"         => $nrorg
        );
        $this->composicaoFactory->replicarAlteracaoIdAtivo($params);
    }
}

?>