<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Factory\SmAssinaturaDigitalFactory as AssinaturaFactory;
use Teknisa\Libs\Util\SMTP as SMTPService;
use Teknisa\SM\Util\Utilities;
use Teknisa\SM\Factory\Sm00005000ParametrizacoesServicesFactory;
use Teknisa\Libs\Util\ConnectionCustom;
use Teknisa\Libs\Util\NewCode;
//require_once "/home/developer/workfolder/sm/backend/vendor/pear/http_request2/HTTP/Request2.php";
require_once "HTTP/Request2.php";

class SmAssinaturaDigitalService {

    protected $entityManager;
    protected $connection;
    protected $environment;
    protected $assinaturaFactory;
    protected $urlIntegracaoCertisign;
    protected $smtpService;
    protected $parametrizacoesServicesFactory;
    protected $connectionCustom;
    protected $newCode;
    

    public function __construct(EntityManager $entityManager, Environment $environment, $urlIntegracaoCertisign, ConnectionCustom $connectionCustom) {
        $this->entityManager          = $entityManager;
        $this->environment            = $environment;
        $this->urlIntegracaoCertisign = $urlIntegracaoCertisign;
        $this->connectionCustom       = $connectionCustom;
        
        $this->iniciarDependencias(null);
    }
    
    private function iniciarDependencias($nrorg) {
        if (is_null($nrorg))
            $nrorg = $this->environment->getNrOrg();
            
        $this->setConnection($nrorg);
        $this->assinaturaFactory = new AssinaturaFactory($this->entityManager);
        $this->parametrizacoesServicesFactory = new Sm00005000ParametrizacoesServicesFactory($this->entityManager);
        $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
    }
    
    private function setConnection($nrorg) {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection    = $novoEntityManager->getConnection();
        }
        else {
            $this->connection = $this->entityManager->getConnection();
        }
    }

    public function convertPDFinBase64($documento){
        try {

            $handle = fopen($documento,'r');
            $file_content = fread($handle,filesize($documento));
            fclose($handle);

            // $byte_array = unpack("C*", $file_content);

            $encoded = base64_encode($file_content);

            return $encoded;

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function buildJsontoUpload($nmDocumento, $documento){
        
        $base64 = $this->convertPDFinBase64($documento);

        $arrayupload = array(
            "fileName" => $nmDocumento,
            "bytes"  => $base64
        );

        $jsonUpload = json_encode($arrayupload, JSON_PRETTY_PRINT);

        return $jsonUpload;
    }

    public function uploadtoCertsing($nmDocumento, $documento){
        
        $token = $this->getTokenCertsign();
        
        $return = array();
        
        $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/upload', \Http_Request2::METHOD_POST);
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            'Content-Type' => 'application/json',
            'Token' => $token,
        );

        $request->setHeader($headers);

        $parameters = array(
            // Request parameters
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(\Http_Request2::METHOD_POST);

        $body = $this->buildJsontoUpload($nmDocumento, $documento);

        // Request body
        $request->setBody($body);

        try
        {
            $response = $request->send();
            $retorno = json_decode($response->getBody());

            $retorno = json_encode($retorno);
            
            return $retorno;
        }
        catch (\HttpException $ex)
        {
            echo $ex;

            $return['response'] = $ex;

            return $return;
        }
    }

    public function buildJsontoCreate($uploadId, $nmDocumento, $sender, $signers, $eletrSigners){
        
        /* -- Documento -- */ 
        $document = array(
            "name" => $nmDocumento,
            "upload" => array("id" => $uploadId,
                              "name" => $nmDocumento)
            );

        /* -- responsável pelo envio -- */
        $sender = array(
            "name"  => $sender['NOME'],
            "email" => $sender['EMAIL'],
            "individualIdentificationCode" => $sender['CPF']
        );
        
        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "document" => $document,
            "sender"  => $sender,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );

        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);

        return $jsonCreate;
    }

    public function createSignatureCertsing($uploadId, $nmDocumento, $sender, $signers, $eletrSigners){
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/create', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
    
            $url->setQueryVariables($parameters);
    
            $request->setMethod(\Http_Request2::METHOD_POST);
    
            $body = $this->buildJsontoCreate($uploadId, $nmDocumento, $sender, $signers, $eletrSigners);
            $body = str_replace('\/', '/', $body);
            
            $return['request'] = $body;
            // Request body
            $request->setBody($body);
             
            try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function callbackCertsing($filedata, $nrorg) {
        $this->iniciarDependencias($nrorg);
        $assinouOuRecusou = null;
        
        try {
            $action = $filedata['action'];
            
            if (($action == 'SIGNATURE-ELETRONIC') || ($action == 'SIGNATURE-DIGITAL')) {
                $assinouOuRecusou = 'A';
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId
                );
                $getSeqPropCont = $this->assinaturaFactory->getSeqPropCont($params);
                
                if (count($getSeqPropCont) > 0) {
                    $documento       = $getSeqPropCont[0]['DOCUMENTO'];
                    $seqPropCont     = $getSeqPropCont[0]['SEQUENCIAL'];
                    $tipo            = $getSeqPropCont[0]['TIPO'];
                    $codigoPropCont  = $getSeqPropCont[0]['CODIGO'];
                    $nmarquivo       = $getSeqPropCont[0]['NMARQUIVO'];
                    $dsmodelodocorca = $getSeqPropCont[0]['DSMODELODOCORCA'];
    
                    if ($tipo == 'PROPOSTA') {
                        
                        $params = array(
                            "NRSEQORCAMENTO"  => $seqPropCont,
                            "NRCPFSIGNATARIO" => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaOrca($params);
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALORCA"];
                    
                        $params = array(
                            "IDSITUACAO"        => 'A',
                            "DSJUSTIFICARECUSA" => null,
                            "NRSEQORCADOC"      => $documento,
                            "NRASSDIGITALORCA"  => $signatario,
                            "NRSEQORCAMENTO"    => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocProposta($params);
                    }
                    else { //$tipo == 'CONTRATO'
                        $params = array(
                            "NRSEQCONTRATO"     => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaContr($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALCONTR"];
                        
                        $params = array(
                            "IDSITUACAO"        => 'A',
                            "DSJUSTIFICARECUSA" => null,
                            "NRSEQCONTDOC"      => $documento,
                            "NRASSDIGITALCONTR" => $signatario,
                            "NRSEQCONTRATO"     => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocContrato($params);
                    }
                    
                    $this->verificaAceitesCallback($seqPropCont, $tipo, $signatario, $documento);
                    $this->enviaEmailCertisign($identifier, $seqPropCont, $tipo, $codigoPropCont, $nmarquivo, $dsmodelodocorca, $assinouOuRecusou, $nrorg, null, $documentId, $documento);
                }
                else {
                    throw new \Exception("Não foi encontrado um documento com o ID informado.");
                }
            }
            else if ($action == 'SIGNATURE-NOK') {
                $assinouOuRecusou = 'R';
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                $message    = $filedata['message'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId
                );
                
                $getSeqPropCont = $this->assinaturaFactory->getSeqPropCont($params);
                
                if (count($getSeqPropCont) > 0) {
                    $documento       = $getSeqPropCont[0]['DOCUMENTO'];
                    $seqPropCont     = $getSeqPropCont[0]['SEQUENCIAL'];
                    $tipo            = $getSeqPropCont[0]['TIPO'];
                    $codigoPropCont  = $getSeqPropCont[0]['CODIGO'];
                    $nmarquivo       = $getSeqPropCont[0]['NMARQUIVO'];
                    $dsmodelodocorca = $getSeqPropCont[0]['DSMODELODOCORCA'];
                    
                    if ($tipo == 'PROPOSTA') {
                        
                        $params = array(
                            "NRSEQORCAMENTO"    => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaOrca($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALORCA"];
                        
                        $params = array(
                            "IDSITUACAO"        => 'R',
                            "DSJUSTIFICARECUSA" => $message,
                            "NRSEQORCADOC"      => $documento,
                            "NRASSDIGITALORCA"  => $signatario,
                            "NRSEQORCAMENTO"    => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocProposta($params);
                    }
                    else { //$tipo == 'CONTRATO'
                         $params = array(
                            "NRSEQCONTRATO"     => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaContr($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALCONTR"];
                        
                        $params = array(
                            "IDSITUACAO"        => 'R',
                            "DSJUSTIFICARECUSA" => $message,
                            "NRSEQCONTDOC"      => $documento,
                            "NRASSDIGITALCONTR" => $signatario,
                            "NRSEQCONTRATO"     => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocContrato($params);
                    }
                    
                    $this->verificaAceitesCallback($seqPropCont, $tipo, $signatario, $documento);
                    $this->enviaEmailCertisign($identifier, $seqPropCont, $tipo, $codigoPropCont, $nmarquivo, $dsmodelodocorca, $assinouOuRecusou, $nrorg, $message, $documentId, $documento);
                }
                else {
                    throw new \Exception("Não foi encontrado um documento com o ID informado.");
                }
            }
        }
        catch(\Exception $e) {
            //file_put_contents("/home/developer/workfolder/TESTE/erro.txt", $e->getMessage());
            
            $nrseqorcamento = null;
            $nrseqcontrato = null;
            $nrseqorcadoc = null;
            $nrseqcontdoc = null;
            
            if (isset($tipo)) {
                if ($tipo == "PROPOSTA") {
                    $nrseqorcamento = $seqPropCont;
                    $nrseqorcadoc = $documento;
                }
                else {
                    $nrseqcontrato = $seqPropCont;
                    $nrseqcontdoc = $documento;
                }
            }
            
            $iddocumcertsign = isset($documentId) ? $documentId : null;
            $nrcpfsignatario = isset($identifier) ? $identifier : null;
            $dserro = "Erro ao atualizar a situação do documento: " . $e->getMessage();
            
            $nrlogassinatdigital = $this->newCode->getCodeByTableName("FCT_LOGASSINATDIGITAL", $nrorg, 1, 10, true);
            
            $params = array(
                "NRLOGASSINATDIGITAL" => $nrlogassinatdigital,
                "NRSEQORCAMENTO"      => $nrseqorcamento,
                "NRSEQCONTRATO"       => $nrseqcontrato,
                "IDDOCUMCERTSIGN"     => $iddocumcertsign,
                "NRCPFSIGNATARIO"     => $nrcpfsignatario,
                "NRSEQORCADOC"        => $nrseqorcadoc,
                "NRSEQCONTDOC"        => $nrseqcontdoc,
                "DSERRO"              => $dserro,
                "NRORG"               => $nrorg
            );
            
            $this->assinaturaFactory->inserirLogAssinaturaDigital($params);
            
            throw $e;
        }
    }
    
    /* Verifica se todos os as situações dos documentos de assinatura para atuzalizar os registros dos signatários. Sendo:
       - Se alguns dos documentos estiver recusado,  o signatário fica como recusado.
       - Fora o caso acima, caso ainda tenha algum documento pendente, o signatário permanece pendente.
       - Caso não haja nenhum pendente e nenhum recusado, o signatário fica aprovado.
       Obs: os dados de assinatura manual serão tratados na própria ação de asssinatura manual.
    */
    public function verificaAceitesCallback($seqPropCont, $tipo, $signatario, $documento) {
        try {
            
            $countAprovado  = 0;
            $countRecusado = 0;
            $countPendente  = 0;
            $justificaRecusa = null;
            
            if ($tipo == 'PROPOSTA') {
                
                $params = array(
                    "NRASSDIGITALORCA" => $signatario,
                    "NRSEQORCAMENTO"   => $seqPropCont
                );
                
                $dadosProposta = $this->assinaturaFactory->recebeDadosAssDocProp($params);  
                
                foreach ($dadosProposta as $dados){
                    
                    if ($dados["IDSITUACAO"] == 'A'){
                        
                        $countAprovado = $countAprovado+1;
                    } 
                    else if ($dados["IDSITUACAO"] == 'R'){ 
                        
                        $countRecusado = $countRecusado+1;
                        
                        if ($justificaRecusa == null) {
                            
                            $justificaRecusa = $dados["DSJUSTIFICARECUSA"];
                        } 
                        else {
                            
                            $justificaRecusa = $justificaRecusa.'    /    '.$dados["DSJUSTIFICARECUSA"];
                        }
                    } 
                    else if ($dados["IDSITUACAO"] == 'P'){
                        
                        $countPendente = $countPendente+1;
                    } 
                } 

                if ($countRecusado > 0) {
                    
                    $params = array(
                        "IDSITUACAO"        => 'R',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALORCA"  => $signatario,
                        "NRSEQORCAMENTO"    => $seqPropCont
                    );
                    
                    $this->assinaturaFactory->updateAssinaProposta($params);
                }
                else if ($countPendente > 0) {
                    
                    /* O registro permanece pendente */
                }
                else {
                    
                     $params = array(
                        "IDSITUACAO"        => 'A',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALORCA"  => $signatario,
                        "NRSEQORCAMENTO"    => $seqPropCont
                    );
                    
                    $this->assinaturaFactory->updateAssinaProposta($params);
                }
            }
            else {
                
                $params = array(
                    "NRASSDIGITALCONTR" => $signatario,
                    "NRSEQCONTRATO" => $seqPropCont
                );
                
                $dadosProposta = $this->assinaturaFactory->recebeDadosAssDocContr($params);  
                
                foreach ($dadosProposta as $dados){
                    
                    if ($dados["IDSITUACAO"] == 'A'){
                        
                        $countAprovado = $countAprovado+1;
                    } 
                    else if ($dados["IDSITUACAO"] == 'R'){ 
                        
                        $countRecusado = $countRecusado+1;
                        
                        if ($justificaRecusa == null) {
                            
                            $justificaRecusa = $dados["DSJUSTIFICARECUSA"];
                        } 
                        else {
                            
                            $justificaRecusa = $justificaRecusa.'<br><br>'.$dados["DSJUSTIFICARECUSA"];
                        }
                    } 
                    else if ($dados["IDSITUACAO"] == 'P'){
                        
                        $countPendente = $countPendente+1;
                    } 
                } 
                
                if ($countRecusado > 0) {
                    
                    $params = array(
                        "IDSITUACAO"        => 'R',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALCONTR"  => $signatario,
                        "NRSEQCONTRATO"    => $seqPropCont
                        
                    );
                    
                    $this->assinaturaFactory->updateAssinaContrato($params);
                }
                else if ($countPendente > 0) {
                    
                    /* O registro permanece pendente */
                }
                else {
                    
                     $params = array(
                        "IDSITUACAO"        => 'A',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALCONTR"  => $signatario,
                        "NRSEQCONTRATO"    => $seqPropCont
                    );
                    
                    $this->assinaturaFactory->updateAssinaContrato($params);
                }
            }
            
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function enviaEmailCertisign($cpfSignatario, $seqPropCont, $tipo, $codigoPropCont, $nmarquivo, $dsmodelodocorca, $assinouOuRecusou, $nrorg, $justificaRecusa, $documentId, $nrseqDocPropCont) {
        $nomeDocumento = $dsmodelodocorca;
        if (empty($nomeDocumento)) {
            $posUltimoPonto = strrpos($nmarquivo, ".");
            $nomeDocumento = substr($nmarquivo, 0, $posUltimoPonto);
        }
        
        try {
            $mailDestinatario = array();
            $mailOperadores = array();
            $dsemailnotifoutros = null;
            
            if ($tipo == 'PROPOSTA') {
                
                $params = array(
                    "NRSEQORCAMENTO"  => $seqPropCont,
                    "NRCPFSIGNATARIO" => $cpfSignatario
                );
                
                $dadosProposta = $this->assinaturaFactory->recebeDadosAssinaturaOrcaMail($params); 
                if (count($dadosProposta) > 0) {
                    
                    $idtestemunha       = $dadosProposta[0]["IDTESTEMUNHA"]; 
                    $nrordenacao        = $dadosProposta[0]["NRORDENACAO"];
                    $nrassdigitalorca   = $dadosProposta[0]["NRASSDIGITALORCA"]; 
                    $dsjustificarecusa  = $dadosProposta[0]["DSJUSTIFICARECUSA"]; 
                    $dsemailnotsignenv  = $dadosProposta[0]["DSEMAILNOTSIGNENV"]; 
                    $dsemailnotifoutros = $dadosProposta[0]["DSEMAILNOTIFOUTROS"]; 
                    $nomeSignatario     = $dadosProposta[0]["NMSIGNATARIO"]; 
                    
                    /* Recebe E-mail Operadores */
                    $params = array(
                        'NRASSDIGITALORCA' => $nrassdigitalorca
                    );
                    
                    $mailOperadores = array();
                    $dataset = $this->assinaturaFactory->recebeEmailOperadoresOrca($params);
                    foreach ($dataset as $row) {
                        $mailOperadores[] = $row["DSEMAILOPER"];
                    }
                    
                    /*recebe Emails Programados*/
                    $maiProgramados = array();
                    if (!empty($dsemailnotsignenv)) {
                        $maiProgramados = $this->recebeEmailsProgramados($seqPropCont, $tipo, $nrassdigitalorca, $idtestemunha, $nrordenacao, $dsemailnotsignenv);
                    }
                }
            }
            else { //$tipo == 'CONTRATO'
                $params = array(
                    "NRSEQCONTRATO"   => $seqPropCont,
                    "NRCPFSIGNATARIO" => $cpfSignatario
                );
                
                $dadosContrato = $this->assinaturaFactory->recebeDadosAssinaturaContrMail($params);  
                if (count($dadosContrato) > 0) {
                    $idtestemunha       = $dadosContrato[0]["IDTESTEMUNHA"]; 
                    $nrordenacao        = $dadosContrato[0]["NRORDENACAO"];
                    $nrassdigitalcont   = $dadosContrato[0]["NRASSDIGITALCONTR"]; 
                    $dsjustificarecusa  = $dadosContrato[0]["DSJUSTIFICARECUSA"]; 
                    $dsemailnotsignenv  = $dadosContrato[0]["DSEMAILNOTSIGNENV"]; 
                    $dsemailnotifoutros = $dadosContrato[0]["DSEMAILNOTIFOUTROS"]; 
                    $nomeSignatario     = $dadosContrato[0]["NMSIGNATARIO"]; 
                    
                    /* Recebe E-mail Operadores */
                    $params = array(
                        'NRASSDIGITALCONTR' => $nrassdigitalcont
                    );
                    
                    $mailOperadores = array();
                    $dataset = $this->assinaturaFactory->recebeEmailOperadoresCont($params);
                    foreach ($dataset as $row) {
                        $mailOperadores[] = $row["DSEMAILOPER"];
                    }
                    
                    /*recebe Emails Programados*/
                    $maiProgramados = array();
                    if (!empty($dsemailnotsignenv)) {
                        $maiProgramados = $this->recebeEmailsProgramados($seqPropCont, $tipo, $nrassdigitalcont, $idtestemunha, $nrordenacao, $dsemailnotsignenv);
                    }
                }
            }
            
            
            if ($dsemailnotifoutros != null) {
                $dsemailnotifoutrosAux = str_replace(" ", "", $dsemailnotifoutros);
                $dsemailnotifoutrosAux = str_replace(",", ";", $dsemailnotifoutrosAux);
                $dsemailnotifoutrosAux = explode(";", $dsemailnotifoutrosAux);
                
                foreach($dsemailnotifoutrosAux as $notifOutros){
                    $mailDestinatario[] = $notifOutros;
                }
            }
            
            $mailDestinatario = array_merge($mailDestinatario, $mailOperadores, $maiProgramados);
            
            if (count($mailDestinatario) > 0) {
                $params = array(
                    "NRORG" => $nrorg
                );
                $configuracaoEmail = $this->parametrizacoesServicesFactory->getConfiguracoesEmail($params)[0];
                
                $paramsSmtp["host"]     = $configuracaoEmail["DSHOSTEMAIL"];
                $paramsSmtp["username"] = $configuracaoEmail["DSUSUARIOEMAIL"];
                $paramsSmtp["password"] = $configuracaoEmail["DSSENHAEMAIL"];
                $paramsSmtp["port"]     = $configuracaoEmail["DSPORTAEMAIL"];
                
                if ($configuracaoEmail["IDSMTPSECURE"] != null){
                    $paramsSmtp["smtpSecure"] = $configuracaoEmail["IDSMTPSECURE"];
                    $paramsSmtp["smtpAuth"] = true;
                }
                
                $textoJustificativaRecusa = null;
                if ($assinouOuRecusou == 'A') {
                    $acao = "assinou";
                }
                else {
                    $acao = "recusou";
                    if (!empty($justificaRecusa)) {
                        $textoJustificativaRecusa = "<p><b>Motivo: </b>$justificaRecusa</p>";
                    }
                }
                
                //Enviar email com $mailDestinatario
                $paramsTemplate = array(
                    "SUBJECT"                 => '',
                    "URL_LOGO_CLIENTE"        => '',
                    "STRING_SUPERIOR_DIREITA" => '',
                    "TITLE_EMAIL"             => 'Notificação de Assinatura',
                    "FOOTER_TEMPLATE"         => '<img src="https://www.teknisa.com/emails/img/rodape-sm.png" width="590" height="84" />',
                    "TEXT_EMAIL"              => '<p><b>'.$nomeSignatario.'</b> '.$acao.' o documento '.$codigoPropCont.' - '.$nomeDocumento.'.</p>' . $textoJustificativaRecusa
                );
                
                $corpo = Utilities::getTemplateEmailTeknisa($paramsTemplate);
                
                $remetente = "sac@teknisa.com";
                $assunto   = "Notificação de Assinatura";
                    
                $this->smtpService  = new SMTPService($paramsSmtp);
                $this->smtpService->setFrom($remetente);
                $this->smtpService->setFromName($remetente);
                $this->smtpService->setSubject($assunto);
                
                $validaEnvioEmail = $this->smtpService->sendMail($corpo, $mailDestinatario);
                if (!$validaEnvioEmail) {
                    throw new \Exception("Falha ao enviar o email.");
                }
            }
        }
        catch(\Exception $e) {
            //file_put_contents("/home/developer/workfolder/TESTE/erro.txt", $e->getMessage());
            
            $nrseqorcamento = null;
            $nrseqcontrato = null;
            $nrseqorcadoc = null;
            $nrseqcontdoc = null;
            
            if ($tipo == "PROPOSTA") {
                $nrseqorcamento = $seqPropCont;
                $nrseqorcadoc = $nrseqDocPropCont;
            }
            else {
                $nrseqcontrato = $seqPropCont;
                $nrseqcontdoc = $nrseqDocPropCont;
            }
            
            $iddocumcertsign = $documentId;
            $nrcpfsignatario = $cpfSignatario;
            $dserro = "Erro ao enviar o email de notificação de assinatura/recusa do documento: " . $e->getMessage();
            
            $nrlogassinatdigital = $this->newCode->getCodeByTableName("FCT_LOGASSINATDIGITAL", $nrorg, 1, 10, true);
            
            $params = array(
                "NRLOGASSINATDIGITAL" => $nrlogassinatdigital,
                "NRSEQORCAMENTO"      => $nrseqorcamento,
                "NRSEQCONTRATO"       => $nrseqcontrato,
                "IDDOCUMCERTSIGN"     => $iddocumcertsign,
                "NRCPFSIGNATARIO"     => $nrcpfsignatario,
                "NRSEQORCADOC"        => $nrseqorcadoc,
                "NRSEQCONTDOC"        => $nrseqcontdoc,
                "DSERRO"              => $dserro,
                "NRORG"               => $nrorg
            );
            
            $this->assinaturaFactory->inserirLogAssinaturaDigital($params);
        }
    }
    
    public function recebeEmailsProgramados($seqPropCont, $tipo, $nrassdigitalOrcaCont, $idtestemunha, $nrordenacao, $dsemailnotsignenv){
        
        $dsemailnotsignenvAux = explode('_',$dsemailnotsignenv);
        $retornoEmail = array();
        
        if ($tipo == 'PROPOSTA') {
            $params = array(
                "NRSEQORCAMENTO" => $seqPropCont
            );
            
            foreach ($dsemailnotsignenvAux as $mailNotifica) {
                
                if ($mailNotifica = 'OPRESP') {
                    $mailOperadorResp = $this->assinaturaFactory->recebeEmailOperadorRespOrca($params);
                    if (count($mailOperadorResp) > 0 && !in_array($mailOperadorResp[0]["DSEMAILOPER"], $retornoEmail)) {
                        $retornoEmail[] = $mailOperadorResp[0]["DSEMAILOPER"];
                    }
                   
                }
                else if ($mailNotifica = 'INT') {
                    $emailInterlocutor = $this->assinaturaFactory->recebeEmailInterlocutorProposta($params);
                    if (count($emailInterlocutor) > 0 && !in_array($emailInterlocutor[0]["EMAILINTERLOC"], $retornoEmail)) {
                        $retornoEmail[] = $email["EMAILINTERLOC"];
                    }
                } 
                else if ($mailNotifica = 'VEND') {
                    $emails = $this->assinaturaFactory->recebeEmailVendedoresProposta($params, $filtro);
                    foreach ($emails as $email) {
                        if (!in_array($email["EMAIL"], $retornoEmail)) {
                            $retornoEmail[] = $email["EMAIL"];
                        }
                    }
                }
                else {
                    if ($mailNotifica = 'SIGTODOS') {
                        $filtro = null;
                    }
                    else if ($mailNotifica = 'SIGANTE') {
                        $filtro = " AND ASS.NRORDENACAO < $nrordenacao ";
                    }
                    else if ($mailNotifica = 'SIGATUAL') {
                       $filtro = " AND ASS.NRORDENACAO = $nrordenacao ";
                    }
                    else if ($mailNotifica = 'SIGPOST') {
                       $filtro = " AND ASS.NRORDENACAO > $nrordenacao ";
                    }
                    
                    $emails = $this->assinaturaFactory->recebeEmailSignatariosProposta($params, $filtro);
                    foreach ($emails as $email) {
                        if (!in_array($email["EMAIL"], $retornoEmail)) {
                            $retornoEmail[] = $email["EMAIL"];
                        }
                    }
                }
            }
        }
        else {
            $params = array(
                "NRSEQCONTRATO" => $seqPropCont
            );
            
            foreach($dsemailnotsignenvAux as $mailNotifica) {
                
                if ($mailNotifica = 'OPRESP') {
                    $mailOperadorResp = $this->assinaturaFactory->recebeEmailOperadorRespCont($params);
                    if (count($mailOperadorResp) > 0) {
                        $retornoEmail[] = $mailOperadorResp[0]["DSEMAILOPER"];
                    }
                   
                }
                else if ($mailNotifica = 'INT') {
                    // Contrato não tem interlocutor
                } 
                else if ($mailNotifica = 'VEND') {
                    // Contrato não tem vendedor
                }
                else {
                    if ($mailNotifica = 'SIGTODOS') {
                        $filtro = null;
                    }
                    else if ($mailNotifica = 'SIGANTE') {
                        $filtro = " AND ASS.NRORDENACAO < $nrordenacao ";
                    }
                    else if ($mailNotifica = 'SIGATUAL') {
                       $filtro = " AND ASS.NRORDENACAO = $nrordenacao ";
                    }
                    else if ($mailNotifica = 'SIGPOST') {
                       $filtro = " AND ASS.NRORDENACAO > $nrordenacao ";
                    }
                    
                    $emails = $this->assinaturaFactory->recebeEmailSignatariosContrato($params, $filtro);
                    foreach ($emails as $email) {
                        if (!in_array($email["EMAIL"], $retornoEmail)) {
                            $retornoEmail[] = $email["EMAIL"];
                        }
                    }
                }
            }
        }
        
        return $retornoEmail;
    }
    
    public function incluirSignatario($documentId, $signers, $eletrSigners) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantAdd', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $body = $this->buildJsontoIncSignatario($documentId, $signers, $eletrSigners);
            
            $return['request'] = $body;
             
            // Request body
            $request->setBody($body);
        
            try {
                $response = $request->send();
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buildJsontoIncSignatario($documentId, $signers, $eletrSigners){

        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "documentId" => $documentId,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );

        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);

        return $jsonCreate;
    }
    
    public function excluirSignatario($documentId, $idsignatcertsign, $nrcpfsignatario) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantDiscard', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documentId" => $documentId,
                "individualIdentificationCode" => $nrcpfsignatario
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request'] = $body;
             
            try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function reenviarSolicitacao ($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/SendReminder', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documents" => [$documentId]
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request'] = $body;
            
            try {
                $response = $request->send();

                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function excluirdocumento($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/delete/'.$documentId, \Http_Request2::METHOD_DELETE);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_DELETE);
            $body = null;
            // Request body
            $request->setBody($body);
            
             try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function dowloadDocumento($chave) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/package?key='.$chave, \Http_Request2::METHOD_GET);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
                'key' => $chave,
                'includeOriginal' => true,
                'includeManifest' => true,
                'zipped' => true,
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_GET);
            $body = null;
            // Request body
            $request->setBody($body);
            
             try {
                $response = $request->send();
                
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getTokenCertsign() {
        
        try {
           
           $nrorg = $this->environment->getNrOrg();
               
           $params = array(
                "NRORG" => $nrorg
           );
            
           $token = $this->assinaturaFactory->getTokenCertsign($params);
        
           $token = $token[0]["DSTOKENCERTSIGN"];   
           
           return $token;    
        
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
}