/* Sistemas Integrados TEKNISA
   Gesto Consolidada

Nome: 9403_TEKSTPO_ETL_SITES.sql
                                                                            
Descrio:                                                                 
Procedure para clculo do ETL de quantidade de sites (unidades) e usurios dos sistemas Teknisa.

Criao:                                                                   
17/02/2011 Ederson Melquades da Cunha e Newton Neves dos Santos

===================
Comandos auxiliares
===================

DROP TABLE ETL_SITES;

CREATE TABLE ETL_SITES
(DTHRCALCULO VARCHAR2(30) null,	
	IDTIPOREGISTRO VARCHAR2(250) null,	
	DTANOREFERENCIA VARCHAR2(4) null,	
	DTMESREFERENCIA VARCHAR2(7) null,	
	NMFANTCLIE VARCHAR2(100) null,	
	NRLICENCAUSO VARCHAR2(10) null,	
	IDMODOLOCAPROC VARCHAR2(30) null,	
	NMLINHAPRODUTO VARCHAR2(250) null,	
	DTINCLUSAOLICENCA VARCHAR2(30) null,	
	DTACEITE VARCHAR2(30) null,	
	DTRESCISAO VARCHAR2(30) null,	
	IDLICENCAATIVA VARCHAR2(30) null,	
	DTVALIDADEPERS VARCHAR2(30) null,	
	IDLICENCAEXCLUIDA VARCHAR2(30) null,	
	IDTIPOLINHAPROD VARCHAR2(30) null,	
	QTSITESVALIDOS NUMBER NULL,
	QTSITESATIVOS NUMBER NULL,
	QTUSUARIOS NUMBER null,	
	QTPTOACESSO NUMBER null,
 QTTOTALUSUARIOS NUMBER null)
/	

SELECT DTHRCALCULO, COUNT(*) FROM ETL_SITES
GROUP BY DTHRCALCULO

DELETE FROM ETL_SITES;

DROP PROCEDURE GERA_ETL_SITES;	

EXECUTE GERA_ETL_SITES;

*/

CREATE OR REPLACE PROCEDURE GERA_ETL_SITES (P_MES IN DATE DEFAULT SYSDATE)
AS
   V_DTHRCALCULO ETL_SITES.DTHRCALCULO%TYPE;
	V_DTMESREFERENCIA ETL_SITES.DTMESREFERENCIA%TYPE;
	V_DTREFERENCIA DATE;
BEGIN
   -- Todos os registros calculados so mantidos para possibilitar comparaes entre os dados durante o tempo
	SELECT TO_CHAR(SYSDATE, 'dy dd/mm/yyyy hh24:mi')
     INTO V_DTHRCALCULO
     FROM DUAL; 
					
	-- A data de referncia  o primeiro dia do ms solicitado, porm os dados so referentes ao ms anterior
	SELECT TO_DATE('01/'||TO_CHAR(NVL(P_MES, SYSDATE), 'MM/YYYY'), 'DD/MM/YYYY') 
	  INTO V_DTREFERENCIA
	  FROM DUAL; 
					
   -- Tratamento para evitar a gravao duplicada para um mesmo ms de referncia, quando for realizada a 
	-- chamada da procedure mais de uma vez no mesmo ms.
	-- *****
	SELECT TO_CHAR(V_DTREFERENCIA-1, 'YYYY/MM')
	  INTO V_DTMESREFERENCIA 
	  FROM DUAL; 
	  
   DELETE 
	  FROM ETL_SITES
    WHERE DTMESREFERENCIA = V_DTMESREFERENCIA;	
	-- *****
	-- *****

   FOR C_DADOS
			 IN (SELECT DECODE(SUM(1) OVER (PARTITION BY IDTIPOREGISTRO, DTANOREFERENCIA, DTMESREFERENCIA, ETL.CDCLIENTE, NRLICENCAUSO
                                       ORDER BY IDTIPOREGISTRO, DTANOREFERENCIA, DTMESREFERENCIA, ETL.CDCLIENTE, NRLICENCAUSO
                      ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW),1,DECODE(IDTIPOREGISTRO, 'UNIDADE SEM LINHA DE PRODUTO RELACIONADA',0,1),0) QTSITESVALIDOS,
               DECODE(SUM(1) OVER (PARTITION BY IDTIPOREGISTRO, DTANOREFERENCIA, DTMESREFERENCIA, ETL.CDCLIENTE, NRLICENCAUSO, QTSITESATIVOS 
                                       ORDER BY IDTIPOREGISTRO, DTANOREFERENCIA, DTMESREFERENCIA, ETL.CDCLIENTE, NRLICENCAUSO, QTSITESATIVOS DESC
                      ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW),1,QTSITESATIVOS,0) QTSITESATIVOS,
															IDTIPOREGISTRO, DTANOREFERENCIA, DTMESREFERENCIA, CLI.NMFANTCLIE||' ('||CLI.CDCLIENTE||')' NMFANTCLIE, NRLICENCAUSO, 
															CLI.CDSEGMER,
															IDMODOLOCAPROC, NMLINHAPRODUTO, DTINCLUSAOLICENCA, 
															DTACEITE, DTRESCISAO, IDLICENCAATIVA, DTVALIDADEPERS, 
															IDLICENCAEXCLUIDA, IDTIPOLINHAPROD, QTUSUARIOS, QTPTOACESSO, QTTOTALUSUARIOS																			
										FROM CLIENTE CLI,
               (-- LINHAS DE PRODUTO DO TIPO "PRODUTO":  considerada a quantidade de usurios da linha de produto
										      SELECT 'CONSIDERADO NO MS (relacionada a linha de produto, existente, vlida, ativa, contrato vigente)' IDTIPOREGISTRO, 
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY') DTANOREFERENCIA,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY/MM') DTMESREFERENCIA,
																							LL.CDCLIENTE, 
                       LL.NRLICENCAUSO, 
																							DECODE(CL.IDMPROCLINPC, 'M', '', 'F', 'Filial-')||DECODE(CL.IDLPROCLINPC, 'M', 'Matriz', 'F', 'Filial') IDMODOLOCAPROC, 
																							CL.NMLINPCLIE||' (cd.:'||CL.CDLINPCLIEINFOR||')' NMLINHAPRODUTO,
																							TO_CHAR(LL.DTINCLUSAO, 'DD/MM/YYYY') DTINCLUSAOLICENCA,
																							TO_CHAR(PR.DTACEPROPFAS, 'DD/MM/YYYY') DTACEITE,
																							TO_CHAR(CL.DTCONTLINPRESC, 'DD/MM/YYYY') DTRESCISAO, 
																							DECODE(NVL(LU.IDLUSOATIV, 'S'), 'S', 'Ativa', 'Inativa') IDLICENCAATIVA, 
																							TO_CHAR(NVL(LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA), 'DD/MM/YYYY') DTVALIDADEPERS, 
																							DECODE(LL.IDLINPEXCLUIDA, 'S', 'Excluda', '') IDLICENCAEXCLUIDA,
                       CASE WHEN (SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, V_DTREFERENCIA-1, 'ATIVA') = 'ATIVA') THEN 1 ELSE 0 END AS QTSITESATIVOS,
																							'PRODUTO' IDTIPOLINHAPROD, 
                       DECODE(CL.IDMPROCLINPC||CL.IDLPROCLINPC, 'FM', 0, CL.NRUSUALINPC) QTUSUARIOS,	
                       0 QTPTOACESSO,
                       DECODE(CL.IDMPROCLINPC||CL.IDLPROCLINPC, 'FM', 0, CL.NRUSUALINPC) QTTOTALUSUARIOS
																		FROM LINPLUSO LL, CONTRATOLINP CL, LICENCAUSO LU, PROPOSTAFAS PR
																	WHERE LL.CDCLIENTE <> 'J26269316000177' -- Teknisa Software
																			AND LL.CDCLIENTE <> 'O00114' -- MIDIATO LTDA
																			AND LL.CDCLIENTE <> 'O99999' -- INATIVO
																			AND LL.CDCLIENTE <> 'F03924269602' -- Teknisa Training
																			AND LL.CDCLIENTE <> 'F54869030659' -- Wilson Lima de Paula
																			AND LL.CDCLIENTE <> 'J00000000000000' -- Sua Empresa
																			AND LL.CDCLIENTE <> 'J01539443000118' -- Winner Academia 
																			AND LL.CDCLIENTE <> 'J06240810000100' -- Teknisa Processamento de Dados - SP
																			AND LL.CDCLIENTE <> 'J05118011000194' -- Teknisa Ltda
																			AND LL.CDCLIENTE <> 'J06348006000130' -- Teknisa Processamento de Dados - BH
																			AND LL.CDCLIENTE <> 'I00000000000132153215' -- Teknisa So Paulo
																			AND LL.CDCLIENTE <> 'I00000000000000067084' -- Teknisa Campinas
																			AND CL.CDCLIENTE = LL.CDCLIENTE
																			AND CL.CDLINPCLIE = LL.CDLINPCLIE
																			AND LU.CDCLIENTE = LL.CDCLIENTE
																			AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
																			AND PR.NRPROPOSFAS = CL.NRPROPOSFAS
																			AND PR.NRVERPROPFAS = CL.NRVERPROPFAS
																			AND CL.IDTIPLINPC IN ('M', 'L')
																			AND LL.DTINCLUSAO < V_DTREFERENCIA
																			AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, V_DTREFERENCIA-1, 'VALIDA') = 'VALIDA'
																			AND (CL.DTCONTLINPRESC IS NULL OR CL.DTCONTLINPRESC >= V_DTREFERENCIA)
																			AND PR.DTACEPROPFAS < V_DTREFERENCIA                  
																	UNION ALL
																-- LINHAS DE PRODUTO DO TIPO "PONTO DE ACESSO":  considerado um usurio por ponto de acesso
																SELECT 'CONSIDERADO NO MS (relacionada a linha de produto, existente, vlida, ativa, contrato vigente)' IDTIPOREGISTRO,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY') DTANOREFERENCIA,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY/MM') DTMESREFERENCIA,
																							LL.CDCLIENTE, 
                       LL.NRLICENCAUSO, 
																							DECODE(CL.IDMPROCLINPC, 'M', '', 'F', 'Filial-')||DECODE(CL.IDLPROCLINPC, 'M', 'Matriz', 'F', 'Filial') IDMODOLOCAPROC, 
																							CL.NMLINPCLIE||' ('||CL.CDLINPCLIEINFOR||') pto.acesso: '||CX.NMCAIXALUSO||' ('||CX.CDCAIXALUSO||')' NMLINHAPRODUTO,
																							TO_CHAR(CX.DTINCLUSAO, 'DD/MM/YYYY') DTINCLUSAOLICENCA,
																							TO_CHAR(PR.DTACEPROPFAS, 'DD/MM/YYYY') DTACEITE,
																							TO_CHAR(CL.DTCONTLINPRESC, 'DD/MM/YYYY') DTRESCISAO, 
																							DECODE(NVL(LU.IDLUSOATIV, 'S'), 'S', 'Ativa', 'Inativa') IDLICENCAATIVA, 
																							TO_CHAR(NVL(CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO), 'DD/MM/YYYY') DTVALIDADEPERS, 
																							DECODE(CX.IDCAIXAEXCLUIDO, 'S', 'Excluda', '') IDLICENCAEXCLUIDA,
                       CASE WHEN (SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, V_DTREFERENCIA-1, 'ATIVA') = 'ATIVA') THEN 1 ELSE 0 END AS QTSITESATIVOS,
																							'PTOACESSO' IDTIPOLINHAPROD, 
                       0 QTUSUARIOS,	
                       1 QTPTOACESSO,
                       1 QTTOTALUSUARIOS
																		FROM LINPLUSO LL, CAIXALUSO CX, CONTRATOLINP CL, LICENCAUSO LU, PROPOSTAFAS PR
																	WHERE LL.CDCLIENTE <> 'J26269316000177' -- Teknisa Software
																			AND LL.CDCLIENTE <> 'O00114' -- MIDIATO LTDA
																			AND LL.CDCLIENTE <> 'O99999' -- INATIVO
																			AND LL.CDCLIENTE <> 'F03924269602' -- Teknisa Training
																			AND LL.CDCLIENTE <> 'F54869030659' -- Wilson Lima de Paula
																			AND LL.CDCLIENTE <> 'J00000000000000' -- Sua Empresa
																			AND LL.CDCLIENTE <> 'J01539443000118' -- Winner Academia 
																			AND LL.CDCLIENTE <> 'J06240810000100' -- Teknisa Processamento de Dados - SP
																			AND LL.CDCLIENTE <> 'J05118011000194' -- Teknisa Ltda
																			AND LL.CDCLIENTE <> 'J06348006000130' -- Teknisa Processamento de Dados - BH
																			AND LL.CDCLIENTE <> 'I00000000000132153215' -- Teknisa So Paulo
																			AND LL.CDCLIENTE <> 'I00000000000000067084' -- Teknisa Campinas
																			AND CX.CDCLIENTE = LL.CDCLIENTE
																			AND CX.NRLICENCAUSO = LL.NRLICENCAUSO
																			AND CL.CDCLIENTE = LL.CDCLIENTE
																			AND CL.CDLINPCLIE = LL.CDLINPCLIE
																			AND LU.CDCLIENTE = LL.CDCLIENTE
																			AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
																			AND PR.NRPROPOSFAS = CL.NRPROPOSFAS
																			AND PR.NRVERPROPFAS = CL.NRVERPROPFAS
																			AND CL.IDTIPLINPC IN ('P', 'A')
																			AND CX.DTINCLUSAO < V_DTREFERENCIA
 																		AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, V_DTREFERENCIA-1, 'VALIDA') = 'VALIDA'
																			AND (CL.DTCONTLINPRESC IS NULL OR CL.DTCONTLINPRESC >= V_DTREFERENCIA)
																			AND PR.DTACEPROPFAS < V_DTREFERENCIA                 
																	UNION ALL
																-- LINHAS DE PRODUTO DO TIPO "PRODUTO":  considerada a quantidade de usurios da linha de produto	
																SELECT 'NO CONSIDERADO' IDTIPOREGISTRO,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY') DTANOREFERENCIA,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY/MM') DTMESREFERENCIA,
																							LL.CDCLIENTE, 
                       LL.NRLICENCAUSO, 
																							DECODE(CL.IDMPROCLINPC, 'M', '', 'F', 'Filial-')||DECODE(CL.IDLPROCLINPC, 'M', 'Matriz', 'F', 'Filial') IDMODOLOCAPROC, 
																							CL.NMLINPCLIE||' (cd.:'||CL.CDLINPCLIEINFOR||')' NMLINHAPRODUTO,
																							TO_CHAR(LL.DTINCLUSAO, 'DD/MM/YYYY') DTINCLUSAOLICENCA,
																							TO_CHAR(PR.DTACEPROPFAS, 'DD/MM/YYYY') DTACEITE,
																							TO_CHAR(CL.DTCONTLINPRESC, 'DD/MM/YYYY') DTRESCISAO, 
																							DECODE(NVL(LU.IDLUSOATIV, 'S'), 'S', 'Ativa', 'Inativa') IDLICENCAATIVA, 
																							TO_CHAR(NVL(LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA), 'DD/MM/YYYY') DTVALIDADEPERS, 
																							DECODE(LL.IDLINPEXCLUIDA, 'S', 'Excluda', '') IDLICENCAEXCLUIDA,
                       CASE WHEN (SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, V_DTREFERENCIA-1, 'ATIVA') = 'ATIVA') THEN 1 ELSE 0 END AS QTSITESATIVOS,
																							'PRODUTO' IDTIPOLINHAPROD, 
                       DECODE(CL.IDMPROCLINPC||CL.IDLPROCLINPC, 'FM', 0, CL.NRUSUALINPC) QTUSUARIOS,	
                       0 QTPTOACESSO,
                       DECODE(CL.IDMPROCLINPC||CL.IDLPROCLINPC, 'FM', 0, CL.NRUSUALINPC) QTTOTALUSUARIOS
																		FROM LINPLUSO LL, CONTRATOLINP CL, LICENCAUSO LU, PROPOSTAFAS PR
																	WHERE LL.CDCLIENTE <> 'J26269316000177' -- Teknisa Software
																			AND LL.CDCLIENTE <> 'O00114' -- MIDIATO LTDA
																			AND LL.CDCLIENTE <> 'O99999' -- INATIVO
																			AND LL.CDCLIENTE <> 'F03924269602' -- Teknisa Training
																			AND LL.CDCLIENTE <> 'F54869030659' -- Wilson Lima de Paula
																			AND LL.CDCLIENTE <> 'J00000000000000' -- Sua Empresa
																			AND LL.CDCLIENTE <> 'J01539443000118' -- Winner Academia 
																			AND LL.CDCLIENTE <> 'J06240810000100' -- Teknisa Processamento de Dados - SP
																			AND LL.CDCLIENTE <> 'J05118011000194' -- Teknisa Ltda
																			AND LL.CDCLIENTE <> 'J06348006000130' -- Teknisa Processamento de Dados - BH
																			AND LL.CDCLIENTE <> 'I00000000000132153215' -- Teknisa So Paulo
																			AND LL.CDCLIENTE <> 'I00000000000000067084' -- Teknisa Campinas
																			AND CL.CDCLIENTE = LL.CDCLIENTE
																			AND CL.CDLINPCLIE = LL.CDLINPCLIE
																			AND LU.CDCLIENTE = LL.CDCLIENTE
																			AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
																			AND PR.NRPROPOSFAS = CL.NRPROPOSFAS
																			AND PR.NRVERPROPFAS = CL.NRVERPROPFAS
																			AND CL.IDTIPLINPC IN ('M', 'L')
                   AND NOT (LL.DTINCLUSAO < V_DTREFERENCIA
                        AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, V_DTREFERENCIA-1, 'VALIDA') = 'VALIDA'
                        AND (CL.DTCONTLINPRESC IS NULL OR CL.DTCONTLINPRESC >= V_DTREFERENCIA)
                        AND PR.DTACEPROPFAS < V_DTREFERENCIA)                  
																	UNION ALL
																SELECT 'NO CONSIDERADO' IDTIPOREGISTRO,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY') DTANOREFERENCIA,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY/MM') DTMESREFERENCIA,
																							LL.CDCLIENTE, 
                       LL.NRLICENCAUSO, 
																							DECODE(CL.IDMPROCLINPC, 'M', '', 'F', 'Filial-')||DECODE(CL.IDLPROCLINPC, 'M', 'Matriz', 'F', 'Filial') IDMODOLOCAPROC, 
																							CL.NMLINPCLIE||' (cd.:'||CL.CDLINPCLIEINFOR||') pto.acesso: '||CX.NMCAIXALUSO||' ('||CX.CDCAIXALUSO||')' NMLINHAPRODUTO,
																							TO_CHAR(CX.DTINCLUSAO, 'DD/MM/YYYY') DTINCLUSAOLICENCA,
																							TO_CHAR(PR.DTACEPROPFAS, 'DD/MM/YYYY') DTACEITE,
																							TO_CHAR(CL.DTCONTLINPRESC, 'DD/MM/YYYY') DTRESCISAO, 
																							DECODE(NVL(LU.IDLUSOATIV, 'S'), 'S', 'Ativa', 'Inativa') IDLICENCAATIVA, 
																							TO_CHAR(NVL(CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO), 'DD/MM/YYYY') DTVALIDADEPERS, 
																							DECODE(CX.IDCAIXAEXCLUIDO, 'S', 'Excluda', '') IDLICENCAEXCLUIDA,
                       CASE WHEN (SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, V_DTREFERENCIA-1, 'ATIVA') = 'ATIVA') THEN 1 ELSE 0 END AS QTSITESATIVOS,
																							'PTOACESSO' IDTIPOLINHAPROD, 
                       0 QTUSUARIOS,	
                       1 QTPTOACESSO,
                       1 QTTOTALUSUARIOS
                  FROM LINPLUSO LL, CAIXALUSO CX, CONTRATOLINP CL, LICENCAUSO LU, PROPOSTAFAS PR
																	WHERE LL.CDCLIENTE <> 'J26269316000177' -- Teknisa Software
																			AND LL.CDCLIENTE <> 'O00114' -- MIDIATO LTDA
																			AND LL.CDCLIENTE <> 'O99999' -- INATIVO
																			AND LL.CDCLIENTE <> 'F03924269602' -- Teknisa Training
																			AND LL.CDCLIENTE <> 'F54869030659' -- Wilson Lima de Paula
																			AND LL.CDCLIENTE <> 'J00000000000000' -- Sua Empresa
																			AND LL.CDCLIENTE <> 'J01539443000118' -- Winner Academia 
																			AND LL.CDCLIENTE <> 'J06240810000100' -- Teknisa Processamento de Dados - SP
																			AND LL.CDCLIENTE <> 'J05118011000194' -- Teknisa Ltda
																			AND LL.CDCLIENTE <> 'J06348006000130' -- Teknisa Processamento de Dados - BH
																			AND LL.CDCLIENTE <> 'I00000000000132153215' -- Teknisa So Paulo
																			AND LL.CDCLIENTE <> 'I00000000000000067084' -- Teknisa Campinas
																			AND CX.CDCLIENTE = LL.CDCLIENTE
																			AND CX.NRLICENCAUSO = LL.NRLICENCAUSO
																			AND CL.CDCLIENTE = LL.CDCLIENTE
																			AND CL.CDLINPCLIE = LL.CDLINPCLIE
																			AND LU.CDCLIENTE = LL.CDCLIENTE
																			AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
																			AND PR.NRPROPOSFAS = CL.NRPROPOSFAS
																			AND PR.NRVERPROPFAS = CL.NRVERPROPFAS
																			AND CL.IDTIPLINPC IN ('P', 'A')
                   AND NOT (CX.DTINCLUSAO < V_DTREFERENCIA
                        AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, V_DTREFERENCIA-1, 'VALIDA') = 'VALIDA'
                        AND (CL.DTCONTLINPRESC IS NULL OR CL.DTCONTLINPRESC >= V_DTREFERENCIA)
                        AND PR.DTACEPROPFAS < V_DTREFERENCIA)                 
																UNION ALL						
                SELECT 'UNIDADE SEM LINHA DE PRODUTO RELACIONADA' IDTIPOREGISTRO,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY') DTANOREFERENCIA,
																							TO_CHAR(V_DTREFERENCIA-1, 'YYYY/MM') DTMESREFERENCIA,
																							LU.CDCLIENTE, 
                       LU.NRLICENCAUSO, 
                       ' ' IDMODOLOCAPROC, 
																							'SEM LINHA DE PRODUTO RELACIONADA' NMLINHAPRODUTO,
																							' ' DTINCLUSAOLICENCA,
																							' ' DTACEITE,
																							' ' DTRESCISAO, 
																							DECODE(NVL(LU.IDLUSOATIV, 'S'), 'S', 'Ativa', 'Inativa') IDLICENCAATIVA, 
																							' ' DTVALIDADEPERS, 
																							' ' IDLICENCAEXCLUIDA,
                       1 QTSITESATIVOS,
																							' ' IDTIPOLINHAPROD, 
																							0 QTUSUARIOS,	
																							0 QTPTOACESSO,
                       0 QTTOTALUSUARIOS
																		FROM LICENCAUSO LU
																	WHERE LU.CDCLIENTE <> 'J26269316000177' -- Teknisa Software
																			AND LU.CDCLIENTE <> 'O00114' -- MIDIATO LTDA
																			AND LU.CDCLIENTE <> 'O99999' -- INATIVO
																			AND LU.CDCLIENTE <> 'F03924269602' -- Teknisa Training
																			AND LU.CDCLIENTE <> 'F54869030659' -- Wilson Lima de Paula
																			AND LU.CDCLIENTE <> 'J00000000000000' -- Sua Empresa
																			AND LU.CDCLIENTE <> 'J01539443000118' -- Winner Academia 
																			AND LU.CDCLIENTE <> 'J06240810000100' -- Teknisa Processamento de Dados - SP
																			AND LU.CDCLIENTE <> 'J05118011000194' -- Teknisa Ltda
																			AND LU.CDCLIENTE <> 'J06348006000130' -- Teknisa Processamento de Dados - BH
																			AND LU.CDCLIENTE <> 'I00000000000132153215' -- Teknisa So Paulo
																			AND LU.CDCLIENTE <> 'I00000000000000067084' -- Teknisa Campinas
                   AND NVL(LU.IDLUSOATIV, 'S') = 'S'         
                   AND LU.DTINCLUSAO < V_DTREFERENCIA
																			AND NOT EXISTS (SELECT LL.CDCLIENTE
																																					FROM LINPLUSO LL, CONTRATOLINP CL, PROPOSTAFAS PR
																																				WHERE LL.CDCLIENTE = LU.CDCLIENTE
																																						AND LL.NRLICENCAUSO = LU.NRLICENCAUSO
                                      AND CL.CDCLIENTE = LL.CDCLIENTE
                                      AND CL.CDLINPCLIE = LL.CDLINPCLIE)
                   AND EXISTS (SELECT CL.CDCLIENTE
                                 FROM PROPOSTAFAS PR, CONTRATOLINP CL
                                WHERE CL.CDCLIENTE = LU.CDCLIENTE
                                  AND PR.NRPROPOSFAS = CL.NRPROPOSFAS
                                  AND PR.NRVERPROPFAS = CL.NRVERPROPFAS
                                  AND PR.DTACEPROPFAS < V_DTREFERENCIA
                                  AND (CL.DTCONTLINPRESC IS NULL OR CL.DTCONTLINPRESC >= V_DTREFERENCIA))) ETL
         WHERE CLI.CDCLIENTE = ETL.CDCLIENTE) LOOP
						INSERT INTO ETL_SITES
									(DTHRCALCULO,	IDTIPOREGISTRO,	DTANOREFERENCIA,	
										DTMESREFERENCIA,	NMFANTCLIE, CDSEGMER,	NRLICENCAUSO,	
										IDMODOLOCAPROC,	NMLINHAPRODUTO,	DTINCLUSAOLICENCA,	
										DTACEITE,	DTRESCISAO,	IDLICENCAATIVA,	
										DTVALIDADEPERS,	IDLICENCAEXCLUIDA,	IDTIPOLINHAPROD,	
          QTSITESVALIDOS, QTSITESATIVOS, QTUSUARIOS,	QTPTOACESSO, QTTOTALUSUARIOS)
						VALUES
									(V_DTHRCALCULO,	C_DADOS.IDTIPOREGISTRO,	C_DADOS.DTANOREFERENCIA,	
										C_DADOS.DTMESREFERENCIA,	C_DADOS.NMFANTCLIE, C_DADOS.CDSEGMER, C_DADOS.NRLICENCAUSO,	
										C_DADOS.IDMODOLOCAPROC,	C_DADOS.NMLINHAPRODUTO,	C_DADOS.DTINCLUSAOLICENCA,	
										C_DADOS.DTACEITE,	C_DADOS.DTRESCISAO,	C_DADOS.IDLICENCAATIVA,	
										NVL(C_DADOS.DTVALIDADEPERS, ' '),	C_DADOS.IDLICENCAEXCLUIDA,	C_DADOS.IDTIPOLINHAPROD,	
          C_DADOS.QTSITESVALIDOS, C_DADOS.QTSITESATIVOS, C_DADOS.QTUSUARIOS,	C_DADOS.QTPTOACESSO, C_DADOS.QTTOTALUSUARIOS);
   END LOOP;
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('GERA_ETL_SITES','5.01.018'); END;
/






